/* Copyright (c) 2003 - Charles Schmidt <cschmidt2@emich.edu> */

#include "Sal-content-handler.h"
#include "Sal-file-dialog.h"
#include "mozilla.h"

#include <nsCOMPtr.h>
#include <nsString.h>
#include <nsReadableUtils.h>
#include <nsIFileURL.h>
#include <nsIURI.h>
#include <nsIURL.h>
#include <nsILocalFile.h>
#include <nsIDOMWindow.h>
#include <nsIDOMWindowInternal.h>
#include <nsIChannel.h>

#include <gtk/gtk.h>


NS_IMPL_ISUPPORTS1(SalamanderContentHandler,nsIHelperAppLauncherDialog)


SalamanderContentHandler::SalamanderContentHandler()
{
	NS_INIT_ISUPPORTS();

}

SalamanderContentHandler::~SalamanderContentHandler()
{

}

/* *****************************
   nsIHelperAppLauncherDialog stubs
   ***************************** */
NS_IMETHODIMP SalamanderContentHandler::Show(nsIHelperAppLauncher *launcher,nsISupports *context)
{
	my_launcher = launcher;
	my_context = context;

	nsCOMPtr<nsIMIMEInfo> mime_info;

	my_launcher->GetMIMEInfo(getter_AddRefs(mime_info));
	mime_info->GetMIMEType(&my_mime_type);
	my_launcher->GetDownloadInfo(getter_AddRefs(my_uri),&my_time_download_started,getter_AddRefs(my_temp_file));

	my_uri->GetSpec(my_url);
	my_uri->GetScheme(my_scheme);

	my_launcher->SaveToDisk(nsnull,PR_FALSE);

	return NS_OK;
}

NS_IMETHODIMP SalamanderContentHandler::PromptForSaveToFile(nsISupports *window_context,const PRUnichar *default_file,const PRUnichar *suggested_file_extension,nsILocalFile **retval)
{
	my_context = window_context;
	nsCOMPtr<nsIDOMWindowInternal> window_internal = do_QueryInterface(window_context);
	nsCOMPtr<nsILocalFile> save_directory;
	gchar *directory_name;

	directory_name = g_strdup(g_get_home_dir());
/* use persistance somehow */
	save_directory = do_CreateInstance(NS_LOCAL_FILE_CONTRACTID);
	save_directory->InitWithPath(NS_ConvertUTF8toUCS2(directory_name));
	g_free(directory_name);

	nsCOMPtr<nsILocalFile> save_file;

	save_file = do_CreateInstance(NS_LOCAL_FILE_CONTRACTID);

	PRInt16 ok_to_save = nsIFilePicker::returnCancel;

	if (ok_to_save == nsIFilePicker::returnCancel) {
		nsCOMPtr<nsIFilePicker> file_picker = do_CreateInstance(S_FILE_DIALOG_CONTRACTID);
		const nsAString &title = NS_ConvertUTF8toUCS2("Save As");

		file_picker->Init(window_internal,PromiseFlatString(title).get(),nsIFilePicker::modeSave);
		file_picker->SetDefaultString(default_file);
		file_picker->SetDisplayDirectory(save_directory);

		file_picker->Show(&ok_to_save);

		if (ok_to_save == nsIFilePicker::returnOK) {
			file_picker->GetFile(getter_AddRefs(save_file));
		}
	}

	if (ok_to_save == nsIFilePicker::returnCancel) {
		return NS_ERROR_FAILURE;
	} else {
		nsCOMPtr<nsIFile> directory;
/* use directory for persistance somehow */
		save_file->GetParent(getter_AddRefs(directory));

		NS_IF_ADDREF(*retval = save_file);

		return NS_OK;
	}
}

NS_METHOD SalamanderContentHandler::ShowProgressDialog(nsIHelperAppLauncher *launcher,nsISupports *context)
{
	return NS_ERROR_NOT_IMPLEMENTED;
}
