/* Copyright (c) 2003 - Charles Schmidt <cschmidt2@emich.edu> */

#include "Sal-progress-dialog.h"
#include "mozilla.h"

//#include "ephy-ellipsizing-label.h"

#include <nsCOMPtr.h>
#include <nsString.h>
#include <nsReadableUtils.h>
#include <nsIMIMEInfo.h>
#include <nsIFTPChannel.h>
#include <nsIChannel.h>

#include <gtk/gtk.h>

static void dialog_close(GtkDialog *dialog,SalamanderProgressDialog *progress);
static void dialog_response(GtkDialog *dialog,gint response,SalamanderProgressDialog *progress);


NS_IMPL_ISUPPORTS4(SalamanderProgressDialog,nsIDownload,nsIWebProgressListener,nsIProgressDialog,nsISupportsWeakReference)

SalamanderProgressDialog::SalamanderProgressDialog()
{
	NS_INIT_ISUPPORTS();
}

SalamanderProgressDialog::~SalamanderProgressDialog()
{

}


NS_METHOD SalamanderProgressDialog::construct_dialog(void)
{
	PRInt64 now = PR_Now();

	my_percent_complete = 0;
	my_interval = 500000;
	my_last_update = now;
	my_start_time = now;
	my_elapsed = now - my_start_time;

	my_canceled = FALSE;

	my_dialog = gtk_dialog_new();

	GtkBox *vbox = GTK_BOX(GTK_DIALOG(my_dialog)->vbox);
	GtkWidget *table;
	GtkSizeGroup *size_group;
	GtkWidget *label;
	GtkWidget *hbox;
	GdkGeometry geom;

	geom.min_width = 0;
	geom.min_height = 0;

	gtk_window_set_position(GTK_WINDOW(my_dialog),GTK_WIN_POS_MOUSE);
	gtk_window_set_geometry_hints(GTK_WINDOW(my_dialog),GTK_WIDGET(my_dialog),&geom,GDK_HINT_MIN_SIZE);
	gtk_window_set_default_size(GTK_WINDOW(my_dialog),375,-1);
	
	gtk_window_set_title(GTK_WINDOW(my_dialog),"Downloading");

	gtk_dialog_add_button(GTK_DIALOG(my_dialog),GTK_STOCK_CANCEL,GTK_RESPONSE_CANCEL);

	hbox = gtk_hbox_new(FALSE,5);
	gtk_box_pack_start(vbox,hbox,FALSE,FALSE,0);

	my_percentage_label = gtk_label_new(NULL);
	gtk_box_pack_start(GTK_BOX(hbox),my_percentage_label,FALSE,FALSE,0);

	my_progress_bar = gtk_progress_bar_new();
	gtk_box_pack_start(GTK_BOX(hbox),my_progress_bar,TRUE,TRUE,0);

	table = gtk_table_new(4,2,FALSE);
	gtk_table_set_col_spacings(GTK_TABLE(table),5);
	gtk_table_set_row_spacings(GTK_TABLE(table),5);
	gtk_box_pack_start(vbox,table,TRUE,TRUE,0);

	size_group = gtk_size_group_new(GTK_SIZE_GROUP_HORIZONTAL);

	label = gtk_label_new(NULL);
	gtk_misc_set_alignment(GTK_MISC(label),0.0,0.5);
	gtk_label_set_markup(GTK_LABEL(label),"<span><b>URI:</b></span>");
	gtk_size_group_add_widget(size_group,label);
	gtk_table_attach(GTK_TABLE(table),label,0,1,0,1,(GtkAttachOptions)0,(GtkAttachOptions)0,0,0);

	gchar *s;
	nsCAutoString tmp;

	my_uri->GetSpec(tmp);
	s = g_strdup(tmp.get());
	//label = ephy_ellipsizing_label_new(s);
	label=gtk_label_new("Start");
	g_free(s);
	//ephy_ellipsizing_label_set_mode(EPHY_ELLIPSIZING_LABEL(label),EPHY_ELLIPSIZE_START);
	gtk_label_set_selectable(GTK_LABEL(label),TRUE);
	gtk_misc_set_alignment(GTK_MISC(label),0.0,0.5);
	gtk_table_attach(GTK_TABLE(table),label,1,2,0,1,(GtkAttachOptions)(GTK_EXPAND | GTK_FILL),(GtkAttachOptions)0,0,0);

	label = gtk_label_new(NULL);
	gtk_misc_set_alignment(GTK_MISC(label),0.0,0.5);
	gtk_label_set_markup(GTK_LABEL(label),"<span><b>Status:</b></span>");
	gtk_size_group_add_widget(size_group,label);
	gtk_table_attach(GTK_TABLE(table),label,0,1,1,2,(GtkAttachOptions)0,(GtkAttachOptions)0,0,0);

	my_status_label = gtk_label_new(NULL);
	gtk_misc_set_alignment(GTK_MISC(my_status_label),0.0,0.5);
	gtk_table_attach(GTK_TABLE(table),my_status_label,1,2,1,2,(GtkAttachOptions)(GTK_EXPAND | GTK_FILL),(GtkAttachOptions)0,0,0);


	label = gtk_label_new(NULL);
	gtk_misc_set_alignment(GTK_MISC(label),0.0,0.5);
	gtk_label_set_markup(GTK_LABEL(label),"<span><b>Time Remaining:</b></span>");
	gtk_size_group_add_widget(size_group,label);
	gtk_table_attach(GTK_TABLE(table),label,0,1,2,3,(GtkAttachOptions)0,(GtkAttachOptions)0,0,0);

	my_time_label = gtk_label_new(NULL);
	gtk_misc_set_alignment(GTK_MISC(my_time_label),0.0,0.5);
	gtk_table_attach(GTK_TABLE(table),my_time_label,1,2,2,3,(GtkAttachOptions)(GTK_EXPAND | GTK_FILL),(GtkAttachOptions)0,0,0);

	g_signal_connect(G_OBJECT(my_dialog),"close",G_CALLBACK(dialog_close),this);
	g_signal_connect(G_OBJECT(my_dialog),"response",G_CALLBACK(dialog_response),this);

	gtk_widget_show_all(my_dialog);


	return NS_OK;
}


nsresult SalamanderProgressDialog::cancel(void)
{
	if (my_canceled == FALSE) {
	        my_action = DOWNLOAD_ACTION_NONE;

		if (my_observer) {
			my_observer->Observe(NS_ISUPPORTS_CAST(nsIProgressDialog*,this),"oncancel",nsnull);
			OnStateChange(nsnull,nsnull,nsIWebProgressListener::STATE_STOP,0);
		}


	        if (my_persist) {
			my_persist->CancelSave();
		}else if (my_launcher) {
			my_launcher->Cancel();
		} else {
			return NS_ERROR_FAILURE;
		}

		my_canceled = TRUE;
	}

	return NS_OK;
}


static void dialog_close(GtkDialog *dialog,SalamanderProgressDialog *progress)
{
	progress->cancel();

	return;
}


static void dialog_response(GtkDialog *dialog,gint response,SalamanderProgressDialog *progress)
{
	if (response == GTK_RESPONSE_CANCEL) {
		progress->cancel();
	}

	return;
}

static gchar * format_time(glong time) {
	gint secs, hours, mins;

	secs = (gint)(time + .5);
	hours = secs / 3600;
	secs -= hours * 3600;
	mins = secs / 60;
	secs -= mins * 60;

	if (hours) {
		return g_strdup_printf("%u:%02u.%02u",hours,mins,secs);
	} else {
		return g_strdup_printf("%02u:%02u",mins,secs);
	}
}


/* *****************************
   nsIProgressDialog stubs
   ***************************** */
NS_IMETHODIMP SalamanderProgressDialog::Init(nsIURI *source,
                                      nsILocalFile *target,
                                      const PRUnichar *display_name,
				      nsIMIMEInfo *mime_info,
                                      PRInt64 start_time,
                                      nsIWebBrowserPersist *persist)
{
	my_uri = source;
	my_file = target;
	my_time_download_started = start_time;
	my_persist = persist;
	my_mime_info = mime_info;
	
	my_action = DOWNLOAD_ACTION_NONE;

	if (mime_info) {
		nsMIMEInfoHandleAction mime_action;

		if (NS_SUCCEEDED(my_mime_info->GetPreferredAction(&mime_action))) {
			
			my_action = (mime_action == nsIMIMEInfo::useHelperApp) ? DOWNLOAD_ACTION_SAVEFORHELPER : DOWNLOAD_ACTION_NONE;
		}
	}

	construct_dialog();

        return NS_OK;
}

NS_IMETHODIMP SalamanderProgressDialog::Open(nsIDOMWindow *parent)
{
	my_parent = parent;

        return NS_OK;
}

NS_IMETHODIMP SalamanderProgressDialog::GetStartTime(PRInt64 *start_time)
{
	*start_time = my_start_time;
	return NS_OK;
}

NS_IMETHODIMP SalamanderProgressDialog::GetSource(nsIURI **source)
{
	NS_IF_ADDREF(*source = my_uri);
        return NS_OK;
}

NS_IMETHODIMP SalamanderProgressDialog::GetTarget(nsILocalFile **target)
{
        nsCOMPtr<nsILocalFile> local_file = do_QueryInterface(my_file);
        NS_IF_ADDREF(*target = local_file);
        return NS_OK;
}

NS_IMETHODIMP SalamanderProgressDialog::GetMIMEInfo(nsIMIMEInfo **mime_info)
{
	NS_IF_ADDREF(*mime_info = my_mime_info);
	return NS_OK;
}

NS_IMETHODIMP SalamanderProgressDialog::GetObserver(nsIObserver **observer)
{
        NS_IF_ADDREF(*observer = my_observer);
        return NS_OK;
}
NS_IMETHODIMP SalamanderProgressDialog::SetObserver(nsIObserver *observer)
{
        my_observer = observer;
        return NS_OK;
}

NS_IMETHODIMP SalamanderProgressDialog::GetListener(nsIWebProgressListener **listener)
{
        *listener = nsnull;
        return NS_OK;
}
NS_IMETHODIMP SalamanderProgressDialog::SetListener(nsIWebProgressListener *listener)
{
        return NS_OK;
}

NS_IMETHODIMP SalamanderProgressDialog::GetPercentComplete(PRInt32 *percent_complete)
{
	*percent_complete = my_percent_complete;
	return NS_OK;
}

NS_IMETHODIMP SalamanderProgressDialog::GetDisplayName(PRUnichar **display_name)
{
	*display_name = nsnull;
        return NS_OK;
}
NS_IMETHODIMP SalamanderProgressDialog::SetDisplayName(const PRUnichar *display_name)
{
        return NS_OK;
}

NS_IMETHODIMP SalamanderProgressDialog::GetPersist(nsIWebBrowserPersist **persist)
{
        NS_IF_ADDREF(*persist = my_persist);
        return NS_OK;
}

NS_IMETHODIMP SalamanderProgressDialog::SetDialog(nsIDOMWindow *dialog)
{
        return NS_OK;
}

NS_IMETHODIMP SalamanderProgressDialog::GetDialog(nsIDOMWindow **dialog)
{
        *dialog = nsnull;
        return NS_OK;
}

NS_IMETHODIMP  SalamanderProgressDialog::GetCancelDownloadOnClose(PRBool *aCancelDownloadOnClose)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

NS_IMETHODIMP  SalamanderProgressDialog::SetCancelDownloadOnClose(PRBool aCancelDownloadOnClose)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}


/* *****************************
   nsIWebProgressListener stubs
   ***************************** */
NS_IMETHODIMP SalamanderProgressDialog::OnStateChange(nsIWebProgress *web_progress,nsIRequest *request,PRUint32 state_flags,PRUint32 status)
{
	if (state_flags & nsIWebProgressListener::STATE_STOP) {
		switch (my_action) {
		case DOWNLOAD_ACTION_SAVEFORHELPER:
g_print("state_stop->download_action_saveforhelper %s:%d\n",__FILE__,__LINE__);
			break;
		case DOWNLOAD_ACTION_NONE:
			if (my_launcher) {
				my_launcher->CloseProgressWindow();
			}
			break;
		case DOWNLOAD_ACTION_OBJECT_NOTIFY:
g_print("state_stop->download_action_notify %s:%d\n",__FILE__,__LINE__);
			break;
		}

		gtk_widget_destroy(my_dialog);
	}

	return NS_OK;
}

NS_IMETHODIMP SalamanderProgressDialog::OnProgressChange(nsIWebProgress *web_progress,nsIRequest *request,PRInt32 current_self_progress,PRInt32 max_self_progress, PRInt32 current_total_progress,PRInt32 max_total_progress)
{
	PRInt64 now = PR_Now();

	if ((now - my_last_update < my_interval) && (max_total_progress != -1) && (current_total_progress < max_total_progress)) {
		return NS_OK;
	}

	my_last_update = now;
	my_elapsed = now - my_start_time;

	PRInt32 current_kbytes = (PRInt32) (current_total_progress / 1024 + 0.5);
	PRInt32 total_kbytes = (PRInt32) (max_total_progress / 1024 + 0.5);

	gfloat progress = -1;
	if (max_total_progress > 0) {
		progress = (gfloat)current_total_progress / (gfloat)max_total_progress;
	}

	gfloat speed = -1;
	PRInt64 current_rate;
	if (my_elapsed) {
		current_rate = ((PRInt64)(current_total_progress)) * 1000000 / my_elapsed;
	} else {
		current_rate = 0;
	}

	speed = ((gfloat)current_rate)/1024;

	gint remaining = -1;
	if (current_rate && (max_total_progress > 0)) {
		remaining = (gint)((max_total_progress - current_total_progress) / current_rate + 0.5);
	}

	gchar *status;

	if (total_kbytes >= 10000) {
		status = g_strdup_printf("%.1fMB / %.1fMB (%.1fKB/sec)",current_kbytes,total_kbytes,speed);
	} else {
		status = g_strdup_printf("%dKB / %dKB (%.1fKB/sec)",current_kbytes,total_kbytes,speed);
	}

	gtk_label_set_text(GTK_LABEL(my_status_label),status);

	gchar *time = format_time(remaining);
	gtk_label_set_text(GTK_LABEL(my_time_label),time);
	g_free(time);

	gchar *progress_s = g_strdup_printf("%2.1f%%",progress > 0 ? progress * 100.0 : 0);
	gtk_label_set_text(GTK_LABEL(my_percentage_label),progress_s);
	g_free(progress_s);

	gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(my_progress_bar),progress);

	return NS_OK;
}

NS_IMETHODIMP SalamanderProgressDialog::OnLocationChange(nsIWebProgress *web_progress,nsIRequest *request,nsIURI *location)
{
	return NS_OK;
}

NS_IMETHODIMP SalamanderProgressDialog::OnStatusChange(nsIWebProgress *web_progress,nsIRequest *request,nsresult status,const PRUnichar *message)
{
	return NS_OK;
}

NS_IMETHODIMP SalamanderProgressDialog:: OnSecurityChange(nsIWebProgress *web_progress,nsIRequest *request,PRUint32 state)
{
	return NS_OK;
}
