
/*mozillaengine.cpp
 *Internal Functions Of Mozilla 1.4b
 */
#include <nsIComponentRegistrar.h>
#include <nsIGenericFactory.h> // generic factory constructor macro
#include<gtk-2.0/gtk/gtk.h>
#include "gtkembedmoz/gtkmozembed.h"
#include <gtkmozembed_internal.h>
#include <unistd.h>
#include<nsIServiceManager.h>
#include<nsIStreamTransportService.h>
#include<nsIPlatformCharset.h>
#include<nsICharsetConverterManager.h>
#include <nsIDOMNSHTMLElement.h>  
#include <nsIDOMNamedNodeMap.h>
#include <nsIDOMMouseEvent.h>
#include <nsIDOMEventTarget.h>
#include "nsIDocShell.h"
#include "nsIPresShell.h"
#include "nsIContentViewer.h"
#include "nsIGlobalHistory.h"
#include "nsIDocShellHistory.h"
#include "nsIWebBrowserFind.h"
#include "nsIWebBrowserFocus.h"
#include "nsIDocument.h"
#include "nsISHEntry.h"
#include "nsISHistoryInternal.h"
#include "nsIHistoryEntry.h"
#include "nsIWebBrowserPrint.h"
#include "nsIURI.h"
#include "nsIMarkupDocumentViewer.h"
#include "nsIComponentManager.h"
#include "nsIScriptGlobalObject.h"
#include "nsIDOMWindowInternal.h"
#include "nsIInterfaceRequestor.h"
#include "nsIFocusController.h"
#include "nsIWebBrowserPersist.h"
#include "nsCWebBrowserPersist.h"
#include "nsNetUtil.h"
#include "nsIChromeEventHandler.h"
#include "nsIClipboardCommands.h"
#include "nsIDOMDocumentStyle.h"
#include "nsIDocShellTreeItem.h"
#include "nsIDocShellTreeNode.h"
#include "nsIDocShellTreeOwner.h"
#include "nsIHTMLContentContainer.h"
#include "nsICSSLoader.h"
#include "nsICSSStyleSheet.h"
#include "nsICSSLoaderObserver.h"
#include "nsIStyleSet.h"
#include "nsIDocumentObserver.h"
#include "nsCWebBrowser.h"
#include "nsReadableUtils.h"
#include "nsUnicharUtils.h"
#include "nsIDOMNSHTMLDocument.h"
#include "nsIDOMHTMLDocument.h"
#include "nsIDOMHTMLCollection.h"
#include "nsIDOMHTMLElement.h"
#include "nsIDeviceContext.h"
#include "nsIPresContext.h"
#include <nsIDOMNSDocument.h>
#include <nsIDOMNSEvent.h>
#include "nsIFilePicker.h"
#include "nsISupportsArray.h"
#include "nsIServiceManager.h"
#include "nsWeakReference.h"

#include "nsString.h"
#include "nsIURI.h"
#include "nsIURL.h"
#include "nsIChannel.h"
#include <mozilla-config.h>

#include "Sal-prompt-service.h"
#include "Sal-progress-dialog.h"
#include "Sal-file-dialog.h"
#include "Sal-content-handler.h"


#define MOZILLA_CONTEXT_NONE     0
#define MOZILLA_CONTEXT_DEFAULT  1
#define MOZILLA_CONTEXT_LINK     2
#define MOZILLA_CONTEXT_IMAGE    4
#define MOZILLA_CONTEXT_DOCUMENT 8
#define MOZILLA_CONTEXT_INPUT    64
#define MOZILLA_CONTEXT_OTHER    128
#define MOZILLA_CONTEXT_XUL      256
#define G_FILEPICKER_CONTRACTID "@mozilla.org/filepicker;1"

NS_GENERIC_FACTORY_CONSTRUCTOR(SalamanderPromptService)
NS_GENERIC_FACTORY_CONSTRUCTOR(SalamanderProgressDialog)
NS_GENERIC_FACTORY_CONSTRUCTOR(SalamanderFileDialog)
NS_GENERIC_FACTORY_CONSTRUCTOR(SalamanderContentHandler)

static const nsModuleComponentInfo components[] = {
	{
		S_PROMPT_SERVICE_CLASSNAME,
		S_PROMPT_SERVICE_CID,
		S_PROMPT_SERVICE_CONTRACTID,
		SalamanderPromptServiceConstructor
	},
	{
		S_PROGRESS_DIALOG_CLASSNAME,
		S_PROGRESS_DIALOG_CID,
		S_PROGRESS_DIALOG_CONTRACTID,
		SalamanderProgressDialogConstructor
	},
	{
		S_PROGRESS_DIALOG_CLASSNAME,
		S_PROGRESS_DIALOG_CID,
		NS_DOWNLOAD_CONTRACTID,
		SalamanderProgressDialogConstructor
	},
	{
		S_FILE_DIALOG_CLASSNAME,
		S_FILE_DIALOG_CID,
		S_FILE_DIALOG_CONTRACTID,
		SalamanderFileDialogConstructor
	},
	{
		S_CONTENT_HANDLER_CLASSNAME,
		S_CONTENT_HANDLER_CID,
		NS_IHELPERAPPLAUNCHERDLG_CONTRACTID,
		SalamanderContentHandlerConstructor
	}
};


extern "C" gboolean mozilla_init(void)
{
	gboolean r = TRUE;
	nsresult rv;
	nsCOMPtr<nsIComponentRegistrar> cr;

	rv = NS_GetComponentRegistrar(getter_AddRefs(cr));
	NS_ENSURE_SUCCESS(rv,rv);

	for (gint i = 0; i < G_N_ELEMENTS(components); i++) {
		nsCOMPtr<nsIGenericFactory> factory;
	
		rv = NS_NewGenericFactory(getter_AddRefs(factory),&(components[i]));
		if (NS_FAILED(rv)) {
			r = FALSE;
			continue;
		}

		rv = cr->RegisterFactory(components[i].mCID,components[i].mDescription,components[i].mContractID,factory);

		if (NS_FAILED(rv)) {
			r = FALSE;
		}
	}

	return r;
}	

extern "C" gboolean mozilla_find(gpointer moz_embed,const gchar *text,gint direction,gboolean wrap,gboolean match_case)
{
	nsCOMPtr<nsIWebBrowser> browser;
	nsresult r;
	PRBool did_find;

	gtk_moz_embed_get_nsIWebBrowser(GTK_MOZ_EMBED(moz_embed),getter_AddRefs(browser));

	if (!browser) {
		g_warning("Could not get nsIWebBrowser from GtkMozEmbed");
		return FALSE;
	}

	nsCOMPtr<nsIWebBrowserFind> finder(do_GetInterface(browser));
	
	finder->SetSearchString((NS_ConvertUTF8toUCS2(text)).get());
	finder->SetFindBackwards((direction >= 1 ? PR_FALSE : PR_TRUE));
	finder->SetWrapFind((wrap == TRUE ? PR_TRUE : PR_FALSE));
	finder->SetEntireWord(PR_FALSE);
	finder->SetMatchCase((match_case == TRUE ? PR_TRUE : PR_FALSE));
	finder->SetSearchFrames(PR_TRUE);

	r = finder->FindNext(&did_find);

	if (!NS_SUCCEEDED(r)) {
		g_warning("no success??\n");
		return FALSE;
	}

	if (did_find == PR_TRUE) {
		return TRUE;
	}

	return FALSE;
}

extern "C" gboolean mozilla_paste_selection (GtkMozEmbed * embed)
{
    nsresult result;
	nsCOMPtr<nsIWebBrowser> browser;

	gtk_moz_embed_get_nsIWebBrowser(embed,getter_AddRefs(browser));
	if (!browser) {
		return FALSE; 
	}
	
	nsCOMPtr < nsIClipboardCommands > clipboard (do_GetInterface (browser));
	return clipboard->Paste();
}

extern "C" gboolean mozilla_copy_selection (GtkMozEmbed * embed)
{
    nsresult result;
	nsCOMPtr<nsIWebBrowser> browser;

	gtk_moz_embed_get_nsIWebBrowser(embed,getter_AddRefs(browser));
	if (!browser) {
		return FALSE; 
	}
	
	nsCOMPtr < nsIClipboardCommands > clipboard (do_GetInterface (browser));
	return clipboard->CopySelection ();
}

extern "C" gboolean mozilla_select_all(GtkMozEmbed *moz)
{
	nsresult result;
	nsCOMPtr<nsIWebBrowser> browser;

	gtk_moz_embed_get_nsIWebBrowser(moz,getter_AddRefs(browser));
	if (!browser) {
		return FALSE;
	}

	nsCOMPtr<nsIClipboardCommands> clipboard(do_GetInterface(browser));
	result = clipboard->SelectAll();
    if (NS_FAILED(result)) {
		return FALSE;
	}
    return TRUE;
}


/* some taken from galeon 1.3.2, thanks guys */
static nsresult get_doc_shell(nsIWebBrowser *browser,nsIDocShell **doc_shell)
{
	nsCOMPtr<nsIDocShellTreeItem> browser_item;

	browser_item = do_QueryInterface(browser);
	if (!browser_item) {
		return NS_ERROR_FAILURE;
	}
      
        // get the owner for that item
        nsCOMPtr<nsIDocShellTreeOwner> tree_owner;
	browser_item->GetTreeOwner(getter_AddRefs(tree_owner));
	if (!tree_owner) {
		return NS_ERROR_FAILURE;
	}

        // get the primary content shell as an item
        nsCOMPtr<nsIDocShellTreeItem> content_item;
        tree_owner->GetPrimaryContentShell(getter_AddRefs(content_item));
	if (!content_item) {
		return NS_ERROR_FAILURE;
	}

        // QI that back to a docshell
        nsCOMPtr<nsIDocShell> doc_shell_ret;
	doc_shell_ret = do_QueryInterface(content_item);
	if (!doc_shell) {
		return NS_ERROR_FAILURE;
	}

	*doc_shell = doc_shell_ret.get();

        NS_IF_ADDREF(*doc_shell);
        
        return NS_OK;
}


extern "C" gboolean mozilla_set_zoom(GtkMozEmbed *moz,gfloat zoom)
{
	nsresult result;
	nsCOMPtr<nsIWebBrowser> browser;
	nsCOMPtr<nsIDocShell> doc_shell;

	gtk_moz_embed_get_nsIWebBrowser(moz,getter_AddRefs(browser));
	if (!browser) {
		return FALSE;
	}

	result = get_doc_shell(browser,getter_AddRefs(doc_shell));
	if (NS_FAILED(result) || !doc_shell) {
		return FALSE;
	}

	nsCOMPtr<nsIContentViewer> content_viewer;	
	result = doc_shell->GetContentViewer(getter_AddRefs(content_viewer));

	if (NS_FAILED(result) || !content_viewer) {
		return FALSE;
	}

	nsCOMPtr<nsIMarkupDocumentViewer> mdv = do_QueryInterface(content_viewer,&result);
	if (NS_FAILED(result) || !mdv) {
		return FALSE;
	}

	result = mdv->SetTextZoom(zoom);

	if (NS_FAILED(result)) {
		return FALSE;
	}

	return TRUE;
}

extern "C" gboolean
mozilla_get_zoom (GtkMozEmbed *embed, float *zoom)
{
    nsresult result;
	nsCOMPtr<nsIWebBrowser> browser;
	nsCOMPtr<nsIDocShell> doc_shell;

	gtk_moz_embed_get_nsIWebBrowser(embed,getter_AddRefs(browser));
	if (!browser) {
		return FALSE;
	}

	result = get_doc_shell(browser,getter_AddRefs(doc_shell));
	if (NS_FAILED(result) || !doc_shell) {
		return FALSE;
	}

	nsCOMPtr<nsIContentViewer> content_viewer;	
	result = doc_shell->GetContentViewer(getter_AddRefs(content_viewer));

	if (NS_FAILED(result) || !content_viewer) {
		return FALSE;
	}

	nsCOMPtr<nsIMarkupDocumentViewer> mdv = do_QueryInterface(content_viewer,&result);
	if (NS_FAILED(result) || !mdv) {
		return FALSE;
	}

	result = mdv->GetTextZoom(zoom);

	if (NS_FAILED(result)) {
		return FALSE;
	}

	return TRUE;
}


/*extern "C" gboolean
mozilla_save (GtkMozEmbed * embed, gchar * file_name, gint all)
{
	
     	gint i = 0;
     	gchar *relative_path = NULL;
		char *filename;
	    nsresult result;
	    nsCOMPtr<nsIWebBrowser> browser;
	    nsCOMPtr<nsIDocShell> doc_shell;
		nsCOMPtr<nsIPresShell> presShell;
		nsCOMPtr<nsIDocument> document;
        nsCOMPtr < nsIDOMDocument > domDoc;
		
    gtk_moz_embed_get_nsIWebBrowser(embed,getter_AddRefs(browser));
	if (!browser) {
		return FALSE;
	}
	result = get_doc_shell(browser,getter_AddRefs(doc_shell));
	if (NS_FAILED(result) || !doc_shell) {
		return FALSE;
	}

	result = doc_shell->GetPresShell (getter_AddRefs (presShell));
    if (NS_FAILED(result) || !presShell) {
	  g_print("error preshell");
		return FALSE;
	}
	result=presShell->GetDocument (getter_AddRefs (document));
	 if (NS_FAILED(result) || !document) {
	  g_print("error document");
		return FALSE;
	}
	
	domDoc = do_QueryInterface (document,&result);
	if (NS_FAILED(result) || !domDoc)
    	g_print("Error en doc");
	 nsCOMPtr<nsIURI> uri;
     result = document->GetDocumentURL (getter_AddRefs(uri));
     if (NS_FAILED(result) || !uri) 
	  g_print("Error");


    nsCOMPtr<nsIWebBrowserPersist> persist(do_QueryInterface(browser));
if (persist) {
	     	nsCOMPtr<nsILocalFile> file;
	     	nsCOMPtr<nsILocalFile> relative = nsnull;
	     	if (all) {
		     	relative_path = g_strdup(file_name);
		     	for (i=strlen(relative_path)-1 ; i >= 0; --i)
		       		if (relative_path[i] == '/') {
				     	relative_path[i] = '\0';
				     	break;
				}
		        nsAutoString s; s.AssignWithConversion(relative_path);
		     	NS_NewLocalFile(s, PR_TRUE, getter_AddRefs(relative));
		       
		} 
	        nsAutoString s; s.AssignWithConversion(file_name);
	     	NS_NewLocalFile(s,PR_TRUE,getter_AddRefs(file));

	     if (file)
			result=persist->SaveDocument(domDoc,file,relative, nsnull, 0, 0);
	       	if (NS_FAILED(result) ) {
		     g_print("error al grabar");
			 return FALSE;
	}
	     	if (all) g_free(relative_path);
	     	return ( TRUE );
	}
     	return ( FALSE );
}*/

static nsIDocShell *
mozilla_get_docshell (GtkMozEmbed * b)
{
	nsresult result;

	/* get nsIBrowser */
	nsIWebBrowser *wb = nsnull;
	gtk_moz_embed_get_nsIWebBrowser (b, &wb);
	nsCOMPtr < nsIDocShell > docShell;

	nsCOMPtr < nsIDocShellTreeItem > browserAsItem;
	browserAsItem = do_QueryInterface (wb);
	if (!browserAsItem)
		return NULL;
	// get the tree owner for that item
	nsCOMPtr < nsIDocShellTreeOwner > treeOwner;
	result = browserAsItem->GetTreeOwner (getter_AddRefs (treeOwner));
	if (!NS_SUCCEEDED (result) || !treeOwner)
		return NULL;
	// get the primary content shell as an item
	nsCOMPtr < nsIDocShellTreeItem > contentItem;
	result = treeOwner->
		GetPrimaryContentShell (getter_AddRefs (contentItem));
	if (!NS_SUCCEEDED (result) || !contentItem)
		return NULL;
	// QI that back to a docshell
	docShell = do_QueryInterface (contentItem);

	return docShell;
}

static nsIPresShell *
mozilla_get_presShell (nsIDocShell * docShell)
{
	nsresult result;

	nsCOMPtr < nsIPresShell > presShell;
	result = docShell->GetPresShell (getter_AddRefs (presShell));
	if (!NS_SUCCEEDED (result) || !presShell)
		return NULL;
	return presShell;
}

static nsIDOMDocument *
mozilla_get_dom_document (nsIPresShell * presShell)
{
	nsCOMPtr < nsIDocument > document;
	presShell->GetDocument (getter_AddRefs (document));

	/* get nsIDOMDocument */
	nsCOMPtr < nsIDOMDocument > domDoc = do_QueryInterface (document);
	if (!domDoc)
		return NULL;
	return domDoc;
}

extern "C" gboolean
mozilla_save (GtkMozEmbed * embed, gchar * file_name, gint all)
{
	    nsIWebBrowser *wb = nsnull;
     	gint i = 0;
     	gchar *relative_path = NULL;
	    nsresult result;
		
     	gtk_moz_embed_get_nsIWebBrowser(embed,&wb);
     	if (!wb) return( FALSE );
     	nsCOMPtr<nsIDocShell> docShell = mozilla_get_docshell(embed);	
     	if (!docShell) return ( FALSE );
     	nsCOMPtr<nsIPresShell> presShell = mozilla_get_presShell(docShell);
     	if (!presShell) return ( FALSE );
     	nsCOMPtr<nsIDOMDocument> domDoc = mozilla_get_dom_document(presShell);
     	if (!domDoc) return ( FALSE );
     	nsCOMPtr<nsIWebBrowserPersist> persist=do_CreateInstance(NS_WEBBROWSERPERSIST_CONTRACTID, &result);
        if (NS_FAILED(result) || !persist) 
		 g_print("error persist");
     	if (persist) {
	     	nsCOMPtr<nsILocalFile> file;
	     	nsCOMPtr<nsILocalFile> relative;
	     	if (all) {
		     relative_path = g_strconcat (file_name, "content", NULL);
             result=NS_NewLocalFile(NS_ConvertUTF8toUCS2(relative_path), PR_TRUE, getter_AddRefs(relative));
		     g_free (relative_path);
		     if (NS_FAILED(result) || !relative)
			 g_print("Error al usar relative %s",file_name);
		} 
	     
			result = NS_NewLocalFile(NS_ConvertUTF8toUCS2(file_name), PR_TRUE, getter_AddRefs(file)); 
        if (NS_FAILED(result) || !file) 
			 g_print("Error al grabar %s",file_name);
   g_print("Legio3");
	     if (file) 
			result=persist->SaveDocument(domDoc,file,relative, nsnull, 0, 0);
		 if (NS_FAILED(result)) 
			 g_print("Error al grabar2 %s",file_name);
	     	return ( TRUE );
	}
     	return ( FALSE );
}

extern "C" gint
dom_mouse_click_cb (gpointer dom_event)
{
	int but;
	PRUint16 button;
	
	((nsIDOMMouseEvent *) dom_event)->GetButton (&button);

	but = button;
	printf ("button was %d\n", button);
	if (button == 2)
		g_print ("hola");
	else
		printf ("button was %d\n", button);
	return but;
}

/* Nautilus CREDITS here */
static gchar *
mozilla_get_attribute (nsIDOMNode *node, gchar *attribute)
{
	
    nsresult result;
	
	nsCOMPtr<nsIDOMNamedNodeMap> attributes;
	result = node->GetAttributes(getter_AddRefs (attributes));
	if (!NS_SUCCEEDED (result) || !attributes) return ( NULL );

	nsAutoString attr; 

	attr.AssignWithConversion (attribute);
                                        
	nsCOMPtr<nsIDOMNode> attrNode;
	result = attributes->GetNamedItem (attr, getter_AddRefs (attrNode));
	if (!NS_SUCCEEDED (result) || !attrNode)  return ( NULL );

	nsAutoString nodeValue;
			
	result = attrNode->GetNodeValue (nodeValue);
	if (!NS_SUCCEEDED (result))  return ( NULL );
                                                
	char *cstr = ToNewCString(nodeValue);
	char *lifeSucks = g_strdup (cstr);
			
	nsMemory::Free (cstr);
	
	return ( lifeSucks );
}


extern "C" glong
mozilla_get_context_menu_type(GtkMozEmbed *b, gpointer event,  gchar **img,  gchar **link, gchar **linktext)
{
    
	nsresult result;
	*img = *link = NULL;

	nsIDOMMouseEvent *aMouseEvent =  (nsIDOMMouseEvent*)event;
             
     	nsCOMPtr<nsIDOMEventTarget> OriginalTarget;
//#ifndef CVS_MOZILLA
//     	result = aMouseEvent->GetOriginalTarget(getter_AddRefs(OriginalTarget));
//#else
     	nsCOMPtr<nsIDOMNSEvent> aEvent = do_QueryInterface(aMouseEvent);
     	if (!aEvent) return (MOZILLA_CONTEXT_NONE);	
     	result = aEvent->GetOriginalTarget(getter_AddRefs(OriginalTarget));
//#endif     
     	if (NS_FAILED(result) || !OriginalTarget) return (  MOZILLA_CONTEXT_NONE );
             
     	nsCOMPtr<nsIDOMNode> OriginalNode = do_QueryInterface(OriginalTarget);
     	if (!OriginalNode) return ( MOZILLA_CONTEXT_NONE );
             
     	nsString nodename;
     	OriginalNode->GetNodeName(nodename);
     
     	if (nodename.EqualsWithConversion ("xul:thumb") ||
	    nodename.EqualsWithConversion ("xul:slider"))
       		return (  MOZILLA_CONTEXT_NONE );

     	nsCOMPtr<nsIDOMEventTarget> EventTarget;
     	result = aMouseEvent->GetTarget(getter_AddRefs(EventTarget));
     	if (NS_FAILED(result) || !EventTarget) return (  MOZILLA_CONTEXT_NONE );
             
	nsCOMPtr<nsIDOMNode> node = do_QueryInterface(EventTarget);
	if (!node)    return (  MOZILLA_CONTEXT_NONE );
     
     	nsCOMPtr<nsIDOMDocument> domDoc;
     	result = node->GetOwnerDocument(getter_AddRefs(domDoc));
     	if (!NS_SUCCEEDED (result) || !domDoc) return (  MOZILLA_CONTEXT_NONE );
     
     	nsCOMPtr<nsIDocument> doc = do_QueryInterface(domDoc);
     	if(!doc) return (  MOZILLA_CONTEXT_NONE );
	
     	nsCOMPtr<nsIDOMNSDocument> nsDoc = do_QueryInterface(domDoc);
     	if (!nsDoc) return (  MOZILLA_CONTEXT_NONE );
     
     	nsIURI *baseURI;
     	result = doc->GetBaseURL(baseURI);
     	if (NS_FAILED(result) || !baseURI) return (  MOZILLA_CONTEXT_NONE );

     	nsString mime;
     	nsDoc->GetContentType(mime);
     	if (mime.EqualsWithConversion ("text/xul"))  return (  MOZILLA_CONTEXT_NONE );

	PRUint32 flags = MOZILLA_CONTEXT_NONE;
	nsCOMPtr<nsIDOMHTMLElement> element;
	
     	do {
		PRUint16 type;
		node->GetNodeType(&type);

		element = do_QueryInterface(node);
		if (element)
		{
			nsAutoString tag;
			element->GetTagName(tag);

			if (tag.EqualsWithConversion("input", PR_TRUE))
			{
				flags |= MOZILLA_CONTEXT_INPUT;
			}
			else if (tag.EqualsWithConversion("img", PR_TRUE))
			{
				flags |= MOZILLA_CONTEXT_IMAGE;

				char *src = mozilla_get_attribute (node, "src");
				if (!src)  return (  MOZILLA_CONTEXT_NONE );

			     	nsAutoString srca;
			     	srca.AssignWithConversion(src);
			     
			     	nsCString srcc,imgc;
			     	srcc.AssignWithConversion(srca);
			   
			     	result = baseURI->Resolve(srcc, imgc);
			     	g_free(src);
			   

			     	*img = ToNewCString(imgc);
			   
			     	if (!img) return (MOZILLA_CONTEXT_NONE);
			
		   
			}
		   
			else
			     {
				
				flags |= MOZILLA_CONTEXT_OTHER;
			}

			nsCOMPtr<nsIDOMNamedNodeMap> attributes;
			node->GetAttributes(getter_AddRefs(attributes));
			if (attributes)
			{
				nsCOMPtr<nsIDOMNode> hrefNode;
				nsAutoString href; 
			     	href.AssignWithConversion("href");
				attributes->GetNamedItem(href, getter_AddRefs(hrefNode));
				if (hrefNode)
				{
					flags |= MOZILLA_CONTEXT_LINK;
				     	nsAutoString linkhtml;
				     	nsCOMPtr<nsIDOMNSHTMLElement> nsElement;
				     	
				     	nsElement = do_QueryInterface (element);				      
				     	
				     	if (nsElement)
				       	{
						result = nsElement->GetInnerHTML(linkhtml);
					     	if (NS_SUCCEEDED(result) && ToNewCString(linkhtml)) 
					       		*linktext = g_strdup(ToNewCString(linkhtml));
					}

					char *href =  mozilla_get_attribute (node, "href");
					if (!href)   return (  MOZILLA_CONTEXT_NONE );

				     	nsAutoString hrefa;
				     	hrefa.AssignWithConversion(href);
				     
				     	nsCString hrefc,linkc;
				     	hrefc.AssignWithConversion(hrefa);

				     	result = baseURI->Resolve(hrefc,linkc);
				     	g_free(href);

					*link = ToNewCString(linkc);
				     
					if (!link) return (  MOZILLA_CONTEXT_NONE );

					break;
				}
			}
		}
		nsCOMPtr<nsIDOMNode> parentNode;
		node->GetParentNode(getter_AddRefs(parentNode));
		
		if (!parentNode)
		{
			node = nsnull;
			flags |= MOZILLA_CONTEXT_DOCUMENT;
			break;
		}
		node = parentNode;
	} while (node);

       	return(  flags );
}

extern "C" void
mozilla_save_image (GtkMozEmbed * b, gchar * location, gchar * FullPath)
{
	
     	nsresult rv = NS_OK;
	
     	if (strlen(location) == 0 || !location) return;
     	nsCOMPtr<nsIURI> linkURI;
     	rv = NS_NewURI(getter_AddRefs(linkURI), location);
     	if (NS_FAILED(rv)) return;
     	 nsIWebBrowser *wb = nsnull;
	     gtk_moz_embed_get_nsIWebBrowser(b,&wb);
     	 nsCOMPtr<nsIWebBrowserPersist> persist(do_QueryInterface(wb));
     	if (persist)  {
	     	nsCOMPtr<nsILocalFile> file;
	        nsAutoString s; s.AssignWithConversion(FullPath);
	        NS_NewLocalFile(s, PR_TRUE, getter_AddRefs(file));
	     	if (!file) return;
       		persist->SaveURI(linkURI, nsnull, nsnull, nsnull, nsnull, file);
	}
	return;
}

/**
 * mozilla_unicode_to_locale: Encodes unicode string to something
 * valid for the current locale (which can then be used in GTK+ labels).
 * @uniStr: The unicode string to encode
 */
extern "C" gchar *
mozilla_unicode_to_locale (const PRUnichar *uniStr)
{
        PRInt32 sSize,dSize;
        gchar *output;
        nsAutoString platformCharset;
        nsresult rv;

        /* sanity */
        if (uniStr == NULL)
        {
                return NULL;
        }

        nsCOMPtr<nsIPlatformCharset> svc;
        nsCOMPtr<nsICharsetConverterManager> ccm;
        nsCOMPtr<nsIUnicodeEncoder> encoder;
        const nsString str (uniStr);

        svc = do_GetService(NS_PLATFORMCHARSET_CONTRACTID, &rv);
        if (NS_SUCCEEDED(rv))
        {
                rv = svc->GetCharset(kPlatformCharsetSel_Menu,
                                     platformCharset);
        }
        if (NS_SUCCEEDED(rv))
        {
                ccm = do_GetService (NS_CHARSETCONVERTERMANAGER_CONTRACTID,
                                     &rv);
        }
        if (NS_SUCCEEDED(rv))
        {
                rv = ccm->GetUnicodeEncoder(&platformCharset,
                                            getter_AddRefs(encoder));
        }
        if (NS_SUCCEEDED(rv)) {
                sSize = str.Length ();
                encoder->GetMaxLength (str.get(), sSize, &dSize);
                if (dSize)
                {
                        output = g_new0 (gchar, dSize + 1);
                        encoder->Convert (str.get(), &sSize, output, &dSize);
                        encoder->Finish (output, &dSize);
                        encoder->Reset ();
                        return output;
                }
        }
        /* return empty string, if something fail */
        output = g_new0 (gchar, 1);
        *output = '\0';
        return output;
}

extern "C" gchar * 
mozilla_get_document_title (GtkMozEmbed *embed)
{
      PRUnichar *unicode_title;
      gchar *title;

      /* get the title in unicode */
      unicode_title = gtk_moz_embed_get_title_unichar (embed);

      /* attempt conversion */
      title = mozilla_unicode_to_locale (unicode_title);

      /* free unicode version */
      g_free (unicode_title);

      /* return it */
      return title;
}

nsresult SaveLink (GtkMozEmbed * embed, const char *url)
{
	/*nsresult rv;
	nsIWebBrowser *wb = nsnull;
	gtk_moz_embed_get_nsIWebBrowser (embed, &wb);
	nsCOMPtr < nsIDOMWindow > DOMWindow;
    const PRUnichar *aDefaultFile;
	
	rv = wb->GetContentDOMWindow (getter_AddRefs (DOMWindow));
	if (NS_FAILED (rv) || !DOMWindow)
		return NS_ERROR_FAILURE;

	nsCOMPtr < nsIDOMWindowInternal > DOMWInternal =
		do_QueryInterface (DOMWindow);
	if (!DOMWInternal)
		return NS_ERROR_FAILURE;
	
	nsCOMPtr<nsILocalFile> saveDir;
	char *dirName;
	
	dirName = "/home/scipio";
	if (dirName == NULL)
	{
		dirName = g_strdup (g_get_home_dir());
	}

	saveDir = do_CreateInstance (NS_LOCAL_FILE_CONTRACTID);
	saveDir->InitWithPath (NS_ConvertUTF8toUCS2(dirName));
	g_free (dirName);

	nsCOMPtr <nsILocalFile> saveFile (do_CreateInstance(NS_LOCAL_FILE_CONTRACTID));

	PRInt16 okToSave = nsIFilePicker::returnCancel;

	nsCOMPtr<nsIFilePicker> filePicker =
				do_CreateInstance (G_FILEPICKER_CONTRACTID);

	const nsAString &title = NS_ConvertUTF8toUCS2("Select the destination filename");

	filePicker->Init (DOMWInternal ,
			   PromiseFlatString(title).get(), 
			   nsIFilePicker::modeSave);
	filePicker->SetDefaultString (aDefaultFile);
	filePicker->SetDisplayDirectory (saveDir);

	filePicker->Show (&okToSave);

	if (okToSave == nsIFilePicker::returnOK)
	{
		filePicker->GetFile (getter_AddRefs(saveFile));

		nsString uFileName;
		saveFile->GetPath(uFileName);
		const nsCString &aFileName = NS_ConvertUCS2toUTF8(uFileName);

		char *dir = g_path_get_dirname (aFileName.get());

		
		g_free (dir);

		nsCOMPtr<nsIFile> directory;
		rv = saveFile->GetParent (getter_AddRefs(directory));

	
		return NS_OK;
	}
	else
	{
		return NS_ERROR_FAILURE;
	}*/
}


extern "C" gboolean
mozilla_save_link (GtkMozEmbed * embed, const char *url)
{
	nsresult result;

	result = SaveLink (embed, url);
    g_print("kiss");
	return NS_SUCCEEDED (result) ? TRUE : FALSE;
}
