
#include"main.h"

/**
 * misc_string_is_punctuation: tests whether the given char is punctuation
 * or not.
 */
static gboolean
misc_string_is_punctuation (gchar c)
{
	return (c == ' ' || c == '.' || c == '!' || c == '|' ||
		c == ',' || c == ':' || c == ';');
}

/**
 * misc_string_shorten_name: try to shorten a page title (ideally to
 * target_length or less). The heurstics here seems to give pretty good
 * results even down to quite small lengths, generally remaining
 * comprehensible down to around six to eight characters.
 */
/* FIXME: utf8? */
gchar *
misc_string_shorten_name (const gchar *input_name, gint target_length)
{
	gint i, j, length;
	gchar *name;
	gchar *first;
	gchar *str;
	char c;

	if (input_name == NULL)
		return (g_strdup (_("Untitled")));
	
	/* copy and clean name */
	name = g_strdup (input_name);
	g_strstrip (name);
	length = strlen (name);

	
		/* prefixes that only come by themselves */
		if (strncasecmp (name, "index of ", 9) == 0)
		{
			length -= 9;
			memmove(name, name + 9, length + 1);
		}
		else if (strncasecmp (name, "re: ", 4) == 0)
		{
			length -= 4;
			memmove(name, name + 4, length + 1);
		}
		else if (strncasecmp (name, "fwd: ", 5) == 0)
		{
			length -= 5;
			memmove(name, name + 5, length + 1);
		}
		else if (strncasecmp (name, "www.", 4) == 0)
		{
			length -= 4;
			memmove(name, name + 4, length + 1);
		}
		else 
		{
			/* prefixes that can be followed by other
			 * prefixes */
			if (strncasecmp (name, "welcome to ", 11) == 0)
			{
				length -= 11;
				memmove(name, name + 11, length + 1);
			}

			/* prefixes that follow the ones in the
			 * previous group */
			if (strncasecmp (name, "a ", 2) == 0)
			{
				length -= 2;
				memmove(name, name + 2, length + 1);
			}
			else if (strncasecmp (name, "my ", 3) == 0)
			{
				length -= 3;
				memmove(name, name + 3, length + 1);
			}
			else if (strncasecmp (name, "the ", 4) == 0)
			{
				length -= 4;
				memmove(name, name + 4, length + 1);
			}
		}

		/* remove any leading whitespace */
		g_strchug (name);
	

	/* check length */
	length = strlen (name);
	if (length <= target_length)
	{
		return name;
	}

		/* set first to initially point to the terminating '\0'
		 * character */
		first = name + sizeof (gchar) * strlen (name);

		/* search for various separators... we can now search
		 * for ": ", becuase because we have stripped "re:" and
		 * "fwd: " in an earlier test */
		str = strstr (name, " - ");
		if (str != NULL && str < first) first = str;
		str = strstr (name, " -- ");
		if (str != NULL && str < first) first = str;
		str = strstr (name, " | ");
		if (str != NULL && str < first) first = str;
		str = strstr (name, " || ");
		if (str != NULL && str < first) first = str;
		str = strstr (name, ": ");
		if (str != NULL && str < first) first = str;
		str = strstr (name, " :: ");
		if (str != NULL && str < first) first = str;

		*first = '\0';

		/* check length */
		g_strchomp (name);
		length = strlen (name);
		if (length <= target_length)
		{
			return name;
		}
	

	/* getting tricky, try losing a few vowels */
	/* yes, this is very anglocentric -- I don't know of any strategies
	 * for other languages (particularly non-European) -- MattA */
	
		for (i = length - 1, j = length - 1; i >= 0; i--)
		{
			c = name[i];
			/* keep the letter if the string is short enough
			 * already, if the letter is at the beginning of a
			 * word, or if it's not a lowercase vowel */
			if (length <= target_length || i == 0 ||
 			    (i > 0 && name[i - 1] == ' ') ||
 			    /* TRANSLATOR: Put in this string those characters
 			     that can be skipped without making a word totally
 			     irrecognizable. In most languages, those are the
 			     vowels (with and without accents) */
 			    !strchr (_("aeiouyAEIOUY"), c))
			{
				name[j] = c;
				j--;
			}
			else
			{
				length--;
			}
		}
		/* shift */
		memmove(name, name + j + 1, length + 1);

		/* short enough yet? */
		if (length <= target_length)
		{
			return name;
		}
	

	/* argh -- try chopping out whole words */
	for (i = target_length; i > 0; i--)
	{
		if (misc_string_is_punctuation (name[i]))
		{
			while (misc_string_is_punctuation (name[i]))
			{
				i--;
			}
			/* this will do */
			name[i + 1] = '\0';
			g_strchomp (name);
			return name;
		}
	}

	/* just chop off and add ellipsis */
	for (i = 0; i < 3; i++)
	{
		/* don't overflow string length */
		if (name[target_length + i] == '\0')
			break;
		
		name[target_length + i] = '.';
	}
	
	/* terminate string */
	name[target_length + i] = '\0';
	
	/* return it */
	return name;
}